@@ -1,4 +1,5 @@
-from typing import Union
+import sys
+from typing import List, Tuple, Union
 
 import numpy as np
 import scipy.linalg
@@ -8,26 +9,111 @@ import scipy.spatial
 from sklearn.base import BaseEstimator
 from sklearn.utils.validation import check_is_fitted, check_scalar
 
-from datafold.dynfold.base import DmapKernelMethod, TransformType, TSCTransformerMixIn
+from datafold.dynfold.base import TransformType, TSCTransformerMixIn
 from datafold.pcfold import DmapKernelFixed, PCManifold
-from datafold.pcfold.kernels import DmapKernelVariable
+from datafold.pcfold.eigsolver import NumericalMathError, compute_kernel_eigenpairs
+from datafold.pcfold.kernels import DmapKernelVariable, GaussianKernel, PCManifoldKernel
 from datafold.utils.general import (
     diagmat_dot_mat,
     if1dim_colvec,
-    is_float,
-    is_integer,
     mat_dot_diagmat,
     random_subsample,
 )
 
 
-class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
-    """Geometrical parametrization of data manifold based on diffusion processes with
-    (fixed) bandwidth.
+class _DmapKernelAlgorithms:
+    """Collection of re-useable algorithms that appear in models that have a diffusion
+    map kernel.
 
-    The diffusion maps allows to perform non-linear dimension reduction
-    (also known as unsupervervised learning or manifold learning) and also approximates
-    operators
+    See Also
+    --------
+
+    :class:`.DiffusionMaps`
+    :class:`.DiffusionMapsVariable`
+    :class:`.GeometricHarmonicsInterpolator`
+    """
+
+    @staticmethod
+    def solve_eigenproblem(
+        kernel_matrix, n_eigenpairs, is_symmetric, is_stochastic, basis_change_matrix
+    ) -> Tuple[np.ndarray, np.ndarray]:
+
+        try:
+            eigvals, eigvect = compute_kernel_eigenpairs(
+                kernel_matrix=kernel_matrix,
+                n_eigenpairs=n_eigenpairs,
+                is_symmetric=is_symmetric,
+                is_stochastic=is_stochastic,
+                # only normalize after potential basis change
+                normalize_eigenvectors=False,
+                backend="scipy",
+            )
+
+        except NumericalMathError:
+            # re-raise with more details for DMAP kernel
+            raise_numerical_error = True
+        else:
+            max_imag_eigvect = np.abs(np.imag(eigvect)).max()
+            max_imag_eigval = np.abs(np.imag(eigvals)).max()
+
+            if max(max_imag_eigval, max_imag_eigvect) > 1e2 * sys.float_info.epsilon:
+                raise_numerical_error = True
+            else:
+                raise_numerical_error = False
+
+        if raise_numerical_error:
+            raise NumericalMathError(
+                "Eigenpairs have non-negligible imaginary part (larger than "
+                f"{1e2 * sys.float_info.epsilon}. First try to use "
+                f"parameter 'symmetrize_kernel=True' (improves numerical stability) and "
+                f"only if this is not working adjust kernel scale."
+            )
+        else:
+            eigvals = np.real(eigvals)
+            eigvect = np.real(eigvect)
+
+        if basis_change_matrix is not None:
+            eigvect = basis_change_matrix @ eigvect
+
+        eigvect /= np.linalg.norm(eigvect, axis=0)[np.newaxis, :]
+
+        return eigvals, eigvect
+
+    @staticmethod
+    def unsymmetric_kernel_matrix(
+        kernel_matrix: Union[np.ndarray, scipy.sparse.csr_matrix], basis_change_matrix,
+    ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
+        """Transform a kernel matrix obtained from a symmetric conjugate
+        transformation to the diffusion kernel matrix.
+
+        Parameters
+        ----------
+        kernel_matrix
+            Symmetric conjugate kernel matrix of shape `(n_samples, n_samples)`
+
+        basis_change_matrix
+            diagonal elements of basis change matrix
+
+        Returns
+        -------
+        numpy.ndarray
+            Generally non-symmetric matrix of same shape and type as `kernel_matrix`.
+        """
+        inv_basis_change_matrix = scipy.sparse.diags(
+            np.reciprocal(basis_change_matrix.data.ravel())
+        )
+
+        return inv_basis_change_matrix @ kernel_matrix @ inv_basis_change_matrix
+
+
+class DiffusionMaps(BaseEstimator, TSCTransformerMixIn):
+    """Define diffusion processes on point clouds to find meaningful geometric
+    descriptions.
+
+    The model can be used for
+
+    * non-linear dimensionality reduction
+    * approximating eigenfunctions of operators (see ``alpha`` parameter):
 
         - Laplace-Beltrami
         - Fokker-Plank
@@ -35,69 +121,65 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
 
     Parameters
     ----------
-    epsilon
-        Bandwidth/scale of diffusion map kernel (see :py:class:`DmapKernelFixed`).
+    kernel
+        Internal kernel to describe proximity between points. The kernel is passed
+        as an `internal_kernel` to :class:`.DmapKernelFixed`, which describes
+        the diffusion process.
 
     n_eigenpairs
-        Number of eigenpairs to compute from computed diffusion kernel matrix.
+        Number of eigenpairs to compute from kernel matrix.
 
     time_exponent
-        Exponent of eigenvalues as the time progress of the diffusion process.
-
-    cut_off
-        Distance cut off, kernel values with a corresponding larger Euclidean distance
-        are set to zero. Lower values increases the sparsity of kernel matrices and
-        faster computation of eigenpairs at the cost of accuracy.
+        Time of diffusion process (exponent of eigenvalues in embedding).
 
     is_stochastic
-        If True the diffusion kernel matrix is normalized (stochastic rows).
+        If True, the diffusion kernel matrix is normalized (row stochastic).
 
     alpha
-        Re-normalization parameter. Set to `alpha=0` for graph laplacian, `alpha=0.5`
-        Fokker-Plank and `alpha=1` for Laplace-Beltrami (`is_stochastic=True` in all
-        cases).
+        Re-normalization parameter between `(0,1)`. ``alpha=1`` corrects the sampling
+        density in the data as an artifact of the collection process. Special values are
+
+        * `alpha=0` graph Laplacian,
+        * `alpha=0.5` Fokker-Plank operator,
+        * `alpha=1` Laplace-Beltrami operator
+
+        Note, that `is_stochastic=True` is required in all three cases.
 
     symmetrize_kernel
-        If True a conjugate transformation of non-symmetric kernel matrices is performed.
-        This improves numerical stability and allows to use eigensolver algorithms
-        designed for Hermitian matrices.
+        If True, a conjugate transformation is performed if the settings
+        lead to a non-symmetric kernel matrix. This improves numerical stability when
+        solving the eigenvectors of the kernel matrix as it allows algorithms designed
+        for (sparse) Hermitian matrices to be used. If the kernel matrix is symmetric
+        already (`is_stochastic=False`), then the parameter has no effect.
 
-    dist_backend
-        Backend of distance matrix computation. Defaults to `guess_optimal`,
-        which selects the backend based on the selectin of ``cut_off`` and the
-        available algorithms. See also :class:`.DistanceAlgorithm`.
-
-    dist_backend_kwargs,
-        Keyword arguments handled to distance matrix backend.
+    dist_kwargs
+        Keyword arguments passed to the internal distance matrix computation. See
+        :py:meth:`datafold.pcfold.compute_distance_matrix` for parameter arguments.
 
     Attributes
     ----------
     X_: PCManifold
-        Training data during fit, is required for out-of-sample mappings. Equipped with \
-        kernel :py:class:`DmapKernelFixed`
+        Training data during fit. The data is required for out-of-sample mappings.
+        Equipped with kernel :py:class:`DmapKernelFixed`.
 
     eigenvalues_ : numpy.ndarray
-        Eigenvalues of diffusion kernel in decreasing magnitude order.
+        Eigenvalues of diffusion kernel matrix in decreasing order.
 
     eigenvectors_: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-        Eigenvectors of the kernel matrix to parametrizes the data manifold.
+        Eigenvectors of the kernel matrix to parametrize the data manifold.
 
     inv_coeff_matrix_: numpy.ndarray
-        Coeffficient matrix to map points from embedding space back to original space.\
+        Coefficient matrix to map points from embedding space back to original space.\
         Computation is delayed until `inverse_transform` is called for the first time.
 
-    kernel_matrix_ : numpy.ndarray
-        Diffusion kernel matrix computed during fit.
-
-        .. note::
-            Currently, the kernel matrix is only used for testing. It may be removed.
-
+    kernel_matrix_ : Union[numpy.ndarray, scipy.sparse.csr_matrix]
+        Computed kernel matrix, stored only if `store_kernel_matrix=True` is set during
+        fit.
 
     References
     ----------
     :cite:`lafon_diffusion_2004`
     :cite:`coifman_diffusion_2006`
-
     """
 
     _cls_valid_operator_names = (
@@ -109,72 +191,26 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
 
     def __init__(
         self,
-        epsilon: float = 1.0,
+        kernel: PCManifoldKernel = GaussianKernel(epsilon=1.0),
         n_eigenpairs: int = 10,
         time_exponent: float = 0,
-        cut_off: float = np.inf,
         is_stochastic: bool = True,
         alpha: float = 1,
         symmetrize_kernel: bool = True,
-        dist_backend: str = "guess_optimal",
-        dist_backend_kwargs=None,
+        dist_kwargs=None,
     ) -> None:
 
+        self.kernel = kernel
+        self.n_eigenpairs = n_eigenpairs
         self.time_exponent = time_exponent
-
-        super(DiffusionMaps, self).__init__(
-            epsilon=epsilon,
-            n_eigenpairs=n_eigenpairs,
-            cut_off=cut_off,
-            is_stochastic=is_stochastic,
-            alpha=alpha,
-            symmetrize_kernel=symmetrize_kernel,
-            dist_backend=dist_backend,
-            dist_backend_kwargs=dist_backend_kwargs,
-        )
-
-    @classmethod
-    def from_operator_name(cls, name: str, **kwargs) -> "DiffusionMaps":
-        """Instantiate new model to approximate operator (to be selected by name).
-
-        Parameters
-        ----------
-        name
-            "laplace_beltrami", "fokker_plank", "graph_laplacian" or "rbf".
-
-        **kwargs
-            All parameters in :py:class:`DiffusionMaps` but ``alpha``.
-
-        Returns
-        -------
-        DiffusionMaps
-            self
-        """
-
-        if name == "laplace_beltrami":
-            eigfunc_interp = cls.laplace_beltrami(**kwargs)
-        elif name == "fokker_planck":
-            eigfunc_interp = cls.fokker_planck(**kwargs)
-        elif name == "graph_laplacian":
-            eigfunc_interp = cls.graph_laplacian(**kwargs)
-        elif name == "rbf":
-            eigfunc_interp = cls.rbf_kernel(**kwargs)
-        else:
-            raise ValueError(
-                f"name='{name}' not known. Choose from {cls._cls_valid_operator_names}"
-            )
-
-        if name not in cls._cls_valid_operator_names:
-            raise NotImplementedError(
-                f"This is a bug. name={name} each name has to be "
-                f"listed in VALID_OPERATOR_NAMES"
-            )
-
-        return eigfunc_interp
+        self.is_stochastic = is_stochastic
+        self.alpha = alpha
+        self.symmetrize_kernel = symmetrize_kernel
+        self.dist_kwargs = dist_kwargs
 
     @classmethod
     def laplace_beltrami(
-        cls, epsilon=1.0, n_eigenpairs=10, **kwargs,
+        cls, kernel=GaussianKernel(epsilon=1.0), n_eigenpairs=10, **kwargs,
     ) -> "DiffusionMaps":
         """Instantiate new model to approximate Laplace-Beltrami operator.
 
@@ -184,7 +220,7 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
             new instance
         """
         return cls(
-            epsilon=epsilon,
+            kernel=kernel,
             n_eigenpairs=n_eigenpairs,
             is_stochastic=True,
             alpha=1.0,
@@ -192,7 +228,9 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
         )
 
     @classmethod
-    def fokker_planck(cls, epsilon=1.0, n_eigenpairs=10, **kwargs,) -> "DiffusionMaps":
+    def fokker_planck(
+        cls, kernel=GaussianKernel(epsilon=1.0), n_eigenpairs=10, **kwargs,
+    ) -> "DiffusionMaps":
         """Instantiate new model to approximate Fokker-Planck operator.
 
         Returns
@@ -201,7 +239,7 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
             new instance
         """
         return cls(
-            epsilon=epsilon,
+            kernel=kernel,
             n_eigenpairs=n_eigenpairs,
             is_stochastic=True,
             alpha=0.5,
@@ -210,7 +248,7 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
 
     @classmethod
     def graph_laplacian(
-        cls, epsilon=1.0, n_eigenpairs=10, **kwargs,
+        cls, kernel=GaussianKernel(epsilon=1.0), n_eigenpairs=10, **kwargs,
     ) -> "DiffusionMaps":
         """Instantiate new model to approximate graph Laplacian.
 
@@ -220,27 +258,13 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
             new instance
         """
         return cls(
-            epsilon=epsilon,
+            kernel=kernel,
             n_eigenpairs=n_eigenpairs,
             is_stochastic=True,
             alpha=0.0,
             **kwargs,
         )
 
-    @classmethod
-    def rbf_kernel(cls, epsilon=1.0, n_eigenpairs=10, **kwargs,) -> "DiffusionMaps":
-        """Instantiate new model to approximate geometric harmonic functions of
-        radial basis function kernel.
-
-        Returns
-        -------
-        DiffusionMaps
-            new instance
-        """
-        return cls(
-            epsilon=epsilon, n_eigenpairs=n_eigenpairs, is_stochastic=False, **kwargs,
-        )
-
     def _nystrom(self, kernel_cdist, eigvec, eigvals):
         return kernel_cdist @ mat_dot_diagmat(eigvec, np.reciprocal(eigvals))
 
@@ -262,21 +286,21 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
 
         return dmap_embedding
 
-    def _setup_kernel(self):
-        self._kernel = DmapKernelFixed(
-            epsilon=self.epsilon,
-            is_stochastic=self.is_stochastic,
-            alpha=self.alpha,
-            symmetrize_kernel=self.symmetrize_kernel,
-        )
+    def _setup_default_dist_kwargs(self):
+        from copy import deepcopy
 
-    def set_coords(self, indices: np.ndarray) -> "DiffusionMaps":
+        self.dist_kwargs_ = deepcopy(self.dist_kwargs) or {}
+        self.dist_kwargs_.setdefault("cut_off", np.inf)
+        self.dist_kwargs_.setdefault("kmin", 0)
+        self.dist_kwargs_.setdefault("backend", "guess_optimal")
+
+    def set_coords(self, indices: Union[np.ndarray, List[int]]) -> "DiffusionMaps":
         """Set eigenvector coordinates for parsimonious mapping.
 
         Parameters
         ----------
         indices
-            Index values (columns) of ``eigenvalues_`` and ``eigenvectors_`` to keep in
+            Index values of ``eigenvalues_`` and respective ``eigenvectors_`` to keep in
             the model.
 
         Returns
@@ -287,24 +311,30 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
 
         check_is_fitted(self, attributes=["eigenvectors_", "eigenvalues_"])
 
+        indices = np.asarray(indices)
+
         # type hints for mypy
         self.eigenvectors_: np.ndarray = if1dim_colvec(self.eigenvectors_[:, indices])
         self.eigenvalues_: np.ndarray = self.eigenvalues_[indices]
 
         return self
 
-    def fit(self, X: TransformType, y=None, **fit_params) -> "DiffusionMaps":
-        """Compute diffusion kernel matrix and its' eigenpairs for manifold
-        parametrization.
+    def fit(
+        self, X: TransformType, y=None, store_kernel_matrix=False
+    ) -> "DiffusionMaps":
+        """Compute diffusion kernel matrix and its' eigenpairs.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Training data with shape `(n_samples, n_features)`.
+            Training data of shape `(n_samples, n_features)`.
         
         y: None
             ignored
 
+        store_kernel_matrix
+            If True, store the kernel matrix in attribute ``kernel_matrix_``.
+
         Returns
         -------
         DiffusionMaps
@@ -317,47 +347,64 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
             X, features_out=[f"dmap{i}" for i in range(self.n_eigenpairs)]
         )
 
-        self._setup_kernel()
+        self._setup_default_dist_kwargs()
 
-        # Need to hold X in class to be able to compute cdist distance matrix which is
-        # required for out-of-sample transforms
-        self.X_ = PCManifold(
-            X,
-            kernel=self.kernel_,
-            cut_off=self.cut_off,
-            dist_backend=self.dist_backend,
-            **(self.dist_backend_kwargs or {}),
+        # Note the DmapKernel is a kernel that wraps another kernel to provides the
+        # DMAP specific functionality.
+        self._dmap_kernel = DmapKernelFixed(
+            internal_kernel=self.kernel,
+            is_stochastic=self.is_stochastic,
+            alpha=self.alpha,
+            symmetrize_kernel=self.symmetrize_kernel,
         )
 
-        # basis_change_matrix is None if not required
-        # save kernel_matrix for now to use it for testing, but it may not be necessary
-        # for larger problems
-        (
-            self.kernel_matrix_,
-            _basis_change_matrix,
-            self._row_sums_alpha,
-        ) = self.X_.compute_kernel_matrix()
+        # Need to hold X in class to be able to compute cdist distance matrix during
+        # out-of-sample transforms.
+        self.X_ = PCManifold(
+            X, kernel=self._dmap_kernel, dist_kwargs=self.dist_kwargs_,
+        )
 
-        self.eigenvalues_, self.eigenvectors_ = self._solve_eigenproblem(
-            self.kernel_matrix_, _basis_change_matrix
+        kernel_output = self.X_.compute_kernel_matrix()
+        (
+            kernel_matrix_,
+            self._cdist_kwargs,
+            ret_extra,
+        ) = PCManifoldKernel.read_kernel_output(kernel_output=kernel_output)
+
+        # if key is not present, this is a bug. The value for the key can also be None.
+        basis_change_matrix = ret_extra["basis_change_matrix"]
+
+        (
+            self.eigenvalues_,
+            self.eigenvectors_,
+        ) = _DmapKernelAlgorithms.solve_eigenproblem(
+            kernel_matrix=kernel_matrix_,
+            n_eigenpairs=self.n_eigenpairs,
+            is_symmetric=self._dmap_kernel.is_symmetric,
+            is_stochastic=self.is_stochastic,
+            basis_change_matrix=basis_change_matrix,
         )
 
         self.eigenvectors_ = self._same_type_X(
-            X, values=self.eigenvectors_, set_columns=self.features_out_[1]
+            X, values=self.eigenvectors_, feature_names=self.features_out_[1]
         )
 
-        if self.kernel_.is_symmetric_transform(is_pdist=True):
-            self.kernel_matrix_ = self._unsymmetric_kernel_matrix(
-                kernel_matrix=self.kernel_matrix_,
-                basis_change_matrix=_basis_change_matrix,
+        if self._dmap_kernel.is_symmetric_transform(is_pdist=True):
+            kernel_matrix_ = _DmapKernelAlgorithms.unsymmetric_kernel_matrix(
+                kernel_matrix=kernel_matrix_, basis_change_matrix=basis_change_matrix,
             )
 
+        if store_kernel_matrix:
+            self.kernel_matrix_ = kernel_matrix_
+
         return self
 
     def transform(self, X: TransformType) -> TransformType:
-        r"""Map out-of-sample points into embedding space with Nyström extension.
 
-        From solving the eigenproblem of the kernel diffusion matrix :math:`K`
+        r"""Embed out-of-sample points with Nyström extension.
+
+        From solving the eigenproblem of the diffusion kernel :math:`K`
+        (:class:`.DmapKernelFixed`)
 
         .. math::
             K(X,X) \Psi = \Psi \Lambda
@@ -369,25 +416,28 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
 
         where :math:`K(X, Y)` is a component-wise evaluation of the kernel.
 
+        Note, that the Nyström mapping can be used for image mappings irrespective of
+        whether the computed kernel matrix :math:`K(X,X)` is symmetric.
+        For details on this see :cite:`fernandez_diffusion_2015` (especially equation 5).
+
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Out-of-sample points with shape `(n_samples, n_features)` to map to embedding
-            space.
+            Data points of shape `(n_samples, n_features)` to be embedded.
 
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type as `X` with shape `(n_samples, n_coords)`
+            same type as `X` of shape `(n_samples, n_coords)`
         """
-
-        check_is_fitted(self, ("X_", "eigenvalues_", "eigenvectors_", "kernel_"))
+        check_is_fitted(self, ("X_", "eigenvalues_", "eigenvectors_"))
 
         X = self._validate_data(X, validate_array_kwargs=dict(ensure_min_samples=1))
         self._validate_feature_input(X, direction="transform")
 
-        kernel_matrix_cdist, _, _ = self.X_.compute_kernel_matrix(
-            X, row_sums_alpha_fit=self._row_sums_alpha
+        kernel_output = self.X_.compute_kernel_matrix(X, **self._cdist_kwargs)
+        kernel_matrix_cdist, _, _ = PCManifoldKernel.read_kernel_output(
+            kernel_output=kernel_output
         )
 
         eigvec_nystroem = self._nystrom(
@@ -399,16 +449,16 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
         dmap_embedding = self._perform_dmap_embedding(eigvec_nystroem)
 
         return self._same_type_X(
-            X, values=dmap_embedding, set_columns=self.features_out_[1]
+            X, values=dmap_embedding, feature_names=self.features_out_[1]
         )
 
     def fit_transform(self, X: TransformType, y=None, **fit_params) -> TransformType:
-        """Fit model and map data directly to embedding space.
+        """Compute diffusion map from data and apply embedding on same data.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Training data with shape `(n_samples, n_features)`
+            Training data of shape `(n_samples, n_features)`
 
         y: None
             ignored
@@ -416,7 +466,7 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type as `X` with shape `(n_samples, n_eigenpairs)`
+            same type as `X` of shape `(n_samples, n_eigenpairs)`
         """
 
         X = self._validate_data(X, validate_array_kwargs=dict(ensure_min_samples=2))
@@ -424,26 +474,26 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
 
         dmap_embedding = self._perform_dmap_embedding(self.eigenvectors_)
         return self._same_type_X(
-            X, values=dmap_embedding, set_columns=self.features_out_[1]
+            X, values=dmap_embedding, feature_names=self.features_out_[1]
         )
 
     def inverse_transform(self, X: TransformType) -> TransformType:
-        """Map points from embedding space back to original (ambient) space.
+        """Pre-image from embedding space back to original (ambient) space.
 
         .. note::
-            Currently, this is only  a linear map in a least squares sense. Overwrite
-            this function for more advanced inverse mappings.
+            Currently, this is only a linear map in a least squares sense. Overwrite
+            this function for more advanced pre-image mappings.
 
         Parameters
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            Out-of-sample points with shape `(n_samples, n_coords)` to
+            Out-of-sample data of shape `(n_samples, n_coords)` to
             map from embedding space to original space.
 
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type as `X` with shape (`n_samples, n_features)`
+            same type as `X` of shape (`n_samples, n_features)`
         """
 
         check_is_fitted(self)
@@ -457,11 +507,11 @@ class DiffusionMaps(DmapKernelMethod, TSCTransformerMixIn):
 
         X_orig_space = np.asarray(X) @ self.inv_coeff_matrix_
         return self._same_type_X(
-            X, values=X_orig_space, set_columns=self.features_in_[1]
+            X, values=X_orig_space, feature_names=self.features_in_[1]
         )
 
 
-class DiffusionMapsVariable(DmapKernelMethod, TSCTransformerMixIn):
+class DiffusionMapsVariable(TSCTransformerMixIn):
     """(experimental, not documented)
     .. warning::
         This class is not documented. Contributions are welcome
@@ -483,34 +533,26 @@ class DiffusionMapsVariable(DmapKernelMethod, TSCTransformerMixIn):
         expected_dim=2,
         beta=-0.5,
         symmetrize_kernel=False,
-        dist_backend="brute",
-        dist_backend_kwargs=None,
+        dist_kwargs=None,
     ):
-
+        self.epsilon = epsilon
+        self.n_eigenpairs = n_eigenpairs
         self.expected_dim = expected_dim
         self.beta = beta
         self.nn_bandwidth = nn_bandwidth
 
         # TODO: To implement: cut_off: float = np.inf (allow also sparsity!)
-        super(DiffusionMapsVariable, self).__init__(
-            epsilon=epsilon,
-            n_eigenpairs=n_eigenpairs,
-            cut_off=None,
-            is_stochastic=False,
-            alpha=-1,
-            symmetrize_kernel=symmetrize_kernel,
-            dist_backend=dist_backend,
-            dist_backend_kwargs=dist_backend_kwargs,
-        )
+        # TODO: generalize DmapKernelVariable also to arbitrary kernels
 
-        self._kernel = DmapKernelVariable(
+        self.dmap_kernel_ = DmapKernelVariable(
             epsilon=self.epsilon,
             k=nn_bandwidth,
             expected_dim=expected_dim,
             beta=beta,
             symmetrize_kernel=symmetrize_kernel,
         )
-        self.alpha = self.kernel_.alpha  # is computed (depends on beta) in kernel
+        self.alpha = self.dmap_kernel_.alpha  # is computed (depends on beta) in kernel
+        self.dist_kwargs = dist_kwargs
 
     @property
     def peq_est_(self):
@@ -532,13 +574,12 @@ class DiffusionMapsVariable(DmapKernelMethod, TSCTransformerMixIn):
             X, features_out=[f"dmap{i}" for i in range(self.n_eigenpairs)]
         )
 
-        pcm = PCManifold(
-            X,
-            kernel=self.kernel_,
-            cut_off=self.cut_off,
-            dist_backend=self.dist_backend,
-            **(self.dist_backend_kwargs or {}),
-        )
+        self.dist_kwargs = self.dist_kwargs or {}
+        self.dist_kwargs.setdefault("cut_off", np.inf)
+        self.dist_kwargs.setdefault("kmin", self.nn_bandwidth)
+        self.dist_kwargs.setdefault("backend", "guess_optimal")
+
+        pcm = PCManifold(X, kernel=self.dmap_kernel_, dist_kwargs=self.dist_kwargs,)
 
         # basis_change_matrix is None if not required
         (
@@ -550,16 +591,23 @@ class DiffusionMapsVariable(DmapKernelMethod, TSCTransformerMixIn):
             self.q_eps_s_,
         ) = pcm.compute_kernel_matrix()
 
-        self.eigenvalues_, self.eigenvectors_ = self._solve_eigenproblem(
-            self.operator_matrix_, _basis_change_matrix
+        (
+            self.eigenvalues_,
+            self.eigenvectors_,
+        ) = _DmapKernelAlgorithms.solve_eigenproblem(
+            kernel_matrix=self.operator_matrix_,
+            n_eigenpairs=self.n_eigenpairs,
+            is_symmetric=self.dmap_kernel_.is_symmetric,
+            is_stochastic=True,
+            basis_change_matrix=_basis_change_matrix,
         )
 
         # TODO: note here the kernel is actually NOT the kernel but the operator matrix
         #  ("L") -> see "Variable bandwidth diffusion maps" by Berry et al.
         #  Maybe think about a way to transform this?
 
-        if self.kernel_.is_symmetric_transform(is_pdist=True):
-            self.operator_matrix_ = self._unsymmetric_kernel_matrix(
+        if self.dmap_kernel_.is_symmetric_transform(is_pdist=True):
+            self.operator_matrix_ = _DmapKernelAlgorithms.unsymmetric_kernel_matrix(
                 kernel_matrix=self.operator_matrix_,
                 basis_change_matrix=_basis_change_matrix,
             )
@@ -576,7 +624,7 @@ class DiffusionMapsVariable(DmapKernelMethod, TSCTransformerMixIn):
         )
 
         self.eigenvectors_ = self._same_type_X(
-            X, values=self.eigenvectors_, set_columns=self.features_out_[1]
+            X, values=self.eigenvectors_, feature_names=self.features_out_[1]
         )
 
         return self
@@ -595,11 +643,12 @@ class DiffusionMapsVariable(DmapKernelMethod, TSCTransformerMixIn):
 
 
 class LocalRegressionSelection(BaseEstimator, TSCTransformerMixIn):
-    """Automatic selection of functional independent geometric harmonic vectors for or
-    parsimonious representation.
+    """Automatic selection of functional independent geometric harmonic vectors for
+    parsimonious data manifold embedding.
 
     To measure the functional dependency a local regression regression is performed: The
-    larger the residual between eigenvetor sets the larger the sets are.
+    larger the residuals between eigenvetor sets the more information they add and are
+    therefore more likely to be considered in an embedding.
 
     The kernel used for the local linear regression has a scale of
 
@@ -799,8 +848,8 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixIn):
 
         ######
         # START
-        # The following code was provided by the received code. However, this part
-        # diverges from the paper. Therefore, I use the paper as it also passes the tests.
+        # The following code was provided. However, this part
+        # diverges from the paper. I use the paper version as it also passes the tests.
         ###
         # target_stddev = np.std(target_eigenvector)
         #
@@ -867,7 +916,7 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixIn):
         Parameters
         ----------
         X
-            Eigenvectors with shape `(n_samples, n_eigenvectors)` to make selection on.
+            Eigenvectors of shape `(n_samples, n_eigenvectors)` to make selection on.
 
         Returns
         -------
@@ -929,12 +978,12 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixIn):
         ----------
 
         X
-            Eigenvectors  with shape `(n_samples, n_eigenvectors)` to carry out selection.
+            Eigenvectors of shape `(n_samples, n_eigenvectors)` to carry out selection.
 
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
-            same type as `X` with shape `(n_samples, n_evec_indices)`
+            same type as `X` of shape `(n_samples, n_evec_indices)`
         """
 
         X = self._validate_data(X)
@@ -942,7 +991,9 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixIn):
 
         # choose eigenvectors
         X_selected = self._same_type_X(
-            X, np.asarray(X)[:, self.evec_indices_], set_columns=self.features_out_[1],
+            X,
+            np.asarray(X)[:, self.evec_indices_],
+            feature_names=self.features_out_[1],
         )
 
         return X_selected
